@@ -1,42 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Copyright (C) 2021 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// Copyright (C) 2021 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 #include "qlibrary.h"
 #include "qlibrary_p.h"
 
@@ -68,6 +32,11 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
+Q_TRACE_POINT(qtcore, QLibraryPrivate_load_entry, const QString &fileName);
+Q_TRACE_POINT(qtcore, QLibraryPrivate_load_exit, bool success);
+
 // On Unix systema and on Windows with MinGW, we can mix and match debug and
 // release plugins without problems. (unless compiled in debug-and-release mode
 // - why?)
@@ -209,6 +178,8 @@ static QLibraryScanResult qt_find_pattern(const char *s, qsizetype s_len, QStrin
     return QMachOParser::parse(s, s_len, errMsg);
 #elif defined(Q_OS_WIN)
     return QCoffPeParser::parse({s, s_len}, errMsg);
+#else
+#   warning "Qt does not know how to efficiently parse your platform's binary format; using slow fall-back."
 #endif
     static constexpr auto matcher = [] {
         // QPluginMetaData::MagicString is not NUL-terminated, but
@@ -220,7 +191,7 @@ static QLibraryScanResult qt_find_pattern(const char *s, qsizetype s_len, QStrin
         static_assert(pattern == magic);
         return qMakeStaticByteArrayMatcher(pattern);
     }();
-    qsizetype i = matcher.indexIn(s, s_len);
+    qsizetype i = matcher.indexIn({s, s_len});
     if (i < 0) {
         *errMsg = QLibrary::tr("'%1' is not a Qt plugin").arg(*errMsg);
         return QLibraryScanResult{};
@@ -239,7 +210,7 @@ static QLibraryScanResult qt_find_pattern(const char *s, qsizetype s_len, QStrin
                 information could not be read.
   Returns  true if version information is present and successfully read.
 */
-static bool findPatternUnloaded(const QString &library, QLibraryPrivate *lib)
+static QLibraryScanResult findPatternUnloaded(const QString &library, QLibraryPrivate *lib)
 {
     QFile file(library);
     if (!file.open(QIODevice::ReadOnly)) {
@@ -247,7 +218,7 @@ static bool findPatternUnloaded(const QString &library, QLibraryPrivate *lib)
             lib->errorString = file.errorString();
         qCWarning(qt_lcDebugPlugins, "%ls: cannot open: %ls", qUtf16Printable(library),
                   qUtf16Printable(file.errorString()));
-        return false;
+        return {};
     }
 
     // Files can be bigger than the virtual memory size on 32-bit systems, so
@@ -264,7 +235,7 @@ static bool findPatternUnloaded(const QString &library, QLibraryPrivate *lib)
         // This can't be used as a plugin.
         qCWarning(qt_lcDebugPlugins, "%ls: failed to map to memory: %ls",
                   qUtf16Printable(library), qUtf16Printable(file.errorString()));
-        return false;
+        return {};
     }
 #else
     QByteArray data;
@@ -281,15 +252,19 @@ static bool findPatternUnloaded(const QString &library, QLibraryPrivate *lib)
     QString errMsg = library;
     QLibraryScanResult r = qt_find_pattern(filedata, fdlen, &errMsg);
     if (r.length) {
+#if defined(Q_OF_MACH_O)
+        if (r.isEncrypted)
+            return r;
+#endif
         if (!lib->metaData.parse(QByteArrayView(filedata + r.pos, r.length))) {
             errMsg = lib->metaData.errorString();
-            qCWarning(qt_lcDebugPlugins, "Found invalid metadata in lib %ls: %ls",
+            qCDebug(qt_lcDebugPlugins, "Found invalid metadata in lib %ls: %ls",
                       qUtf16Printable(library), qUtf16Printable(errMsg));
         } else {
             qCDebug(qt_lcDebugPlugins, "Found metadata in lib %ls, metadata=\n%s\n",
                     qUtf16Printable(library),
                     QJsonDocument(lib->metaData.toJson()).toJson().constData());
-            return true;
+            return r;
         }
     } else {
         qCDebug(qt_lcDebugPlugins, "Failed to find metadata in lib %ls: %ls",
@@ -298,7 +273,7 @@ static bool findPatternUnloaded(const QString &library, QLibraryPrivate *lib)
 
     lib->errorString = QLibrary::tr("Failed to extract plugin meta data from '%1': %2")
             .arg(library, errMsg);
-    return false;
+    return {};
 }
 
 static void installCoverageTool(QLibraryPrivate *libPrivate)
@@ -350,9 +325,9 @@ private:
     LibraryMap libraryMap;
 };
 
-static QBasicMutex qt_library_mutex;
-static QLibraryStore *qt_library_data = nullptr;
-static bool qt_library_data_once;
+Q_CONSTINIT static QBasicMutex qt_library_mutex;
+Q_CONSTINIT static QLibraryStore *qt_library_data = nullptr;
+Q_CONSTINIT static bool qt_library_data_once;
 
 QLibraryStore::~QLibraryStore()
 {
@@ -373,12 +348,7 @@ inline void QLibraryStore::cleanup()
             if (lib->libraryUnloadCount.loadRelaxed() > 0) {
                 Q_ASSERT(lib->pHnd.loadRelaxed());
                 lib->libraryUnloadCount.storeRelaxed(1);
-#ifdef __GLIBC__
-                // glibc has a bug in unloading from global destructors
-                // see https://bugzilla.novell.com/show_bug.cgi?id=622977
-                // and http://sourceware.org/bugzilla/show_bug.cgi?id=11941
-                lib->unload(QLibraryPrivate::NoUnloadSys);
-#elif defined(Q_OS_DARWIN)
+#if defined(Q_OS_DARWIN)
                 // We cannot fully unload libraries, as we don't know if there are
                 // lingering references (in system threads e.g.) to Objective-C classes
                 // defined in the library.
@@ -394,7 +364,7 @@ inline void QLibraryStore::cleanup()
 
     // dump all objects that remain
     if (lcDebugLibrary().isDebugEnabled()) {
-        for (QLibraryPrivate *lib : qAsConst(data->libraryMap)) {
+        for (QLibraryPrivate *lib : std::as_const(data->libraryMap)) {
             if (lib)
                 qDebug(lcDebugLibrary)
                         << "On QtCore unload," << lib->fileName << "was leaked, with"
@@ -428,10 +398,12 @@ inline QLibraryPrivate *QLibraryStore::findOrCreate(const QString &fileName, con
     QMutexLocker locker(&qt_library_mutex);
     QLibraryStore *data = instance();
 
+    QString mapName = version.isEmpty() ? fileName : fileName + u'\0' + version;
+
     // check if this library is already loaded
     QLibraryPrivate *lib = nullptr;
     if (Q_LIKELY(data)) {
-        lib = data->libraryMap.value(fileName);
+        lib = data->libraryMap.value(mapName);
         if (lib)
             lib->mergeLoadHints(loadHints);
     }
@@ -440,7 +412,7 @@ inline QLibraryPrivate *QLibraryStore::findOrCreate(const QString &fileName, con
 
     // track this library
     if (Q_LIKELY(data) && !fileName.isEmpty())
-        data->libraryMap.insert(fileName, lib);
+        data->libraryMap.insert(mapName, lib);
 
     lib->libraryRefCount.ref();
     return lib;
@@ -460,9 +432,11 @@ inline void QLibraryStore::releaseLibrary(QLibraryPrivate *lib)
     Q_ASSERT(lib->libraryUnloadCount.loadRelaxed() == 0);
 
     if (Q_LIKELY(data) && !lib->fileName.isEmpty()) {
-        QLibraryPrivate *that = data->libraryMap.take(lib->fileName);
-        Q_ASSERT(lib == that);
-        Q_UNUSED(that);
+        qsizetype n = erase_if(data->libraryMap, [lib](LibraryMap::iterator it) {
+            return it.value() == lib;
+        });
+        Q_ASSERT_X(n, "~QLibrary", "Did not find this library in the library map");
+        Q_UNUSED(n);
     }
     delete lib;
 }
@@ -626,11 +600,11 @@ QtPluginInstanceFunction QLibraryPrivate::loadPlugin()
 bool QLibrary::isLibrary(const QString &fileName)
 {
 #if defined(Q_OS_WIN)
-    return fileName.endsWith(QLatin1String(".dll"), Qt::CaseInsensitive);
+    return fileName.endsWith(".dll"_L1, Qt::CaseInsensitive);
 #else // Generic Unix
 # if defined(Q_OS_DARWIN)
     // On Apple platforms, dylib look like libmylib.1.0.0.dylib
-    if (fileName.endsWith(QLatin1String(".dylib")))
+    if (fileName.endsWith(".dylib"_L1))
         return true;
 # endif
     QString completeSuffix = QFileInfo(fileName).completeSuffix();
@@ -638,25 +612,25 @@ bool QLibrary::isLibrary(const QString &fileName)
         return false;
 
     // if this throws an empty-array error, you need to fix the #ifdef's:
-    const QLatin1String candidates[] = {
+    const QLatin1StringView candidates[] = {
 # if defined(Q_OS_HPUX)
 /*
     See "HP-UX Linker and Libraries User's Guide", section "Link-time Differences between PA-RISC and IPF":
     "In PA-RISC (PA-32 and PA-64) shared libraries are suffixed with .sl. In IPF (32-bit and 64-bit),
     the shared libraries are suffixed with .so. For compatibility, the IPF linker also supports the .sl suffix."
 */
-        QLatin1String("sl"),
+        "sl"_L1,
 #  if defined __ia64
-        QLatin1String("so"),
+        "so"_L1,
 #  endif
 # elif defined(Q_OS_AIX)
-        QLatin1String("a"),
-        QLatin1String("so"),
+        "a"_L1,
+        "so"_L1,
 # elif defined(Q_OS_DARWIN)
-        QLatin1String("so"),
-        QLatin1String("bundle"),
+        "so"_L1,
+        "bundle"_L1,
 # elif defined(Q_OS_UNIX)
-        QLatin1String("so"),
+        "so"_L1,
 # endif
     }; // candidates
 
@@ -739,7 +713,7 @@ void QLibraryPrivate::updatePluginState()
     bool success = false;
 
 #if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
-    if (fileName.endsWith(QLatin1String(".debug"))) {
+    if (fileName.endsWith(".debug"_L1)) {
         // refuse to load a file that ends in .debug
         // these are the debug symbols from the libraries
         // the problem is that they are valid shared library files
@@ -754,7 +728,22 @@ void QLibraryPrivate::updatePluginState()
 
     if (!pHnd.loadRelaxed()) {
         // scan for the plugin metadata without loading
-        success = findPatternUnloaded(fileName, this);
+        QLibraryScanResult result = findPatternUnloaded(fileName, this);
+#if defined(Q_OF_MACH_O)
+        if (result.length && result.isEncrypted) {
+            // We found the .qtmetadata section, but since the library is encrypted
+            // we need to dlopen() it before we can parse the metadata for further
+            // validation.
+            qCDebug(qt_lcDebugPlugins, "Library is encrypted. Doing prospective load before parsing metadata");
+            locker.unlock();
+            load();
+            locker.relock();
+            success = qt_get_metadata(this, &errorString);
+        } else
+#endif
+        {
+            success = result.length != 0;
+        }
     } else {
         // library is already loaded (probably via QLibrary)
         // simply get the target function and call it.
@@ -777,7 +766,7 @@ void QLibraryPrivate::updatePluginState()
     uint qt_version = uint(metaData.value(QtPluginMetaDataKeys::QtVersion).toInteger());
     bool debug = metaData.value(QtPluginMetaDataKeys::IsDebug).toBool();
     if ((qt_version & 0x00ff00) > (QT_VERSION & 0x00ff00) || (qt_version & 0xff0000) != (QT_VERSION & 0xff0000)) {
-        qCWarning(qt_lcDebugPlugins, "In %s:\n"
+        qCDebug(qt_lcDebugPlugins, "In %s:\n"
                  "  Plugin uses incompatible Qt library (%d.%d.%d) [%s]",
                  QFile::encodeName(fileName).constData(),
                  (qt_version&0xff0000) >> 16, (qt_version&0xff00) >> 8, qt_version&0xff,
@@ -787,7 +776,7 @@ void QLibraryPrivate::updatePluginState()
                  QString::number((qt_version & 0xff0000) >> 16),
                  QString::number((qt_version & 0xff00) >> 8),
                  QString::number(qt_version & 0xff),
-                 debug ? QLatin1String("debug") : QLatin1String("release"));
+                 debug ? "debug"_L1 : "release"_L1);
     } else if (PluginMustMatchQtDebug && debug != QtBuildIsDebug) {
         //don't issue a qWarning since we will hopefully find a non-debug? --Sam
         errorString = QLibrary::tr("The plugin '%1' uses incompatible Qt library."
@@ -813,9 +802,11 @@ bool QLibrary::load()
         return false;
     if (d.tag() == Loaded)
         return d->pHnd.loadRelaxed();
-    else
+    if (d->load()) {
         d.setTag(Loaded);
-    return d->load();
+        return true;
+    }
+    return false;
 }
 
 /*!
@@ -943,13 +934,7 @@ QLibrary::~QLibrary()
 
 void QLibrary::setFileName(const QString &fileName)
 {
-    QLibrary::LoadHints lh;
-    if (d) {
-        lh = d->loadHints();
-        d->release();
-        d = {};
-    }
-    d = QLibraryPrivate::findOrCreate(fileName, QString(), lh);
+    setFileNameAndVersion(fileName, QString());
 }
 
 QString QLibrary::fileName() const
@@ -972,13 +957,7 @@ QString QLibrary::fileName() const
 */
 void QLibrary::setFileNameAndVersion(const QString &fileName, int verNum)
 {
-    QLibrary::LoadHints lh;
-    if (d) {
-        lh = d->loadHints();
-        d->release();
-        d = {};
-    }
-    d = QLibraryPrivate::findOrCreate(fileName, verNum >= 0 ? QString::number(verNum) : QString(), lh);
+    setFileNameAndVersion(fileName, verNum >= 0 ? QString::number(verNum) : QString());
 }
 
 /*!
@@ -996,9 +975,10 @@ void QLibrary::setFileNameAndVersion(const QString &fileName, const QString &ver
     if (d) {
         lh = d->loadHints();
         d->release();
-        d = {};
     }
-    d = QLibraryPrivate::findOrCreate(fileName, version, lh);
+    QLibraryPrivate *dd = QLibraryPrivate::findOrCreate(fileName, version, lh);
+    d = dd;
+    d.setTag(isLoaded() ? Loaded : NotLoaded);
 }
 
 /*!
