@@ -1,54 +1,17 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2022 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qplatformdefs.h"
 #include "private/qdatetimeparser_p.h"
 #include "private/qstringiterator_p.h"
 
 #include "qdatastream.h"
-#include "qset.h"
-#include "qlocale.h"
 #include "qdatetime.h"
-#if QT_CONFIG(timezone)
-#include "qtimezone.h"
-#endif
 #include "qdebug.h"
+#include "qlocale.h"
+#include "qset.h"
+#include "qtimezone.h"
+#include "qvarlengtharray.h"
 
 //#define QDATETIMEPARSER_DEBUG
 #if defined (QDATETIMEPARSER_DEBUG) && !defined(QT_NO_DEBUG_STREAM)
@@ -61,6 +24,8 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 template <typename T>
 using ShortVector = QVarLengthArray<T, 13>; // enough for month (incl. leap) and day-of-week names
 
@@ -167,9 +132,7 @@ bool QDateTimeParser::setDigit(QDateTime &v, int index, int newVal) const
     int minute = time.minute();
     int second = time.second();
     int msec = time.msec();
-    Qt::TimeSpec tspec = v.timeSpec();
-    // Only offset from UTC is amenable to setting an int value:
-    int offset = tspec == Qt::OffsetFromUTC ? v.offsetFromUtc() : 0;
+    QTimeZone timeZone = v.timeRepresentation();
 
     const SectionNode &node = sectionNodes.at(index);
     switch (node.type) {
@@ -201,8 +164,8 @@ bool QDateTimeParser::setDigit(QDateTime &v, int index, int newVal) const
     case TimeZoneSection:
         if (newVal < absoluteMin(index) || newVal > absoluteMax(index))
             return false;
-        tspec = Qt::OffsetFromUTC;
-        offset = newVal;
+        // Only offset from UTC is amenable to setting an int value:
+        timeZone = QTimeZone::fromSecondsAheadOfUtc(newVal);
         break;
     case AmPmSection: hour = (newVal == 0 ? hour % 12 : (hour % 12) + 12); break;
     default:
@@ -242,12 +205,7 @@ bool QDateTimeParser::setDigit(QDateTime &v, int index, int newVal) const
     if (!newDate.isValid() || !newTime.isValid())
         return false;
 
-    // Preserve zone:
-    v =
-#if QT_CONFIG(timezone)
-         tspec == Qt::TimeZone ? QDateTime(newDate, newTime, v.timeZone()) :
-#endif
-         QDateTime(newDate, newTime, tspec, offset);
+    v = QDateTime(newDate, newTime, timeZone);
     return true;
 }
 
@@ -264,11 +222,7 @@ int QDateTimeParser::absoluteMax(int s, const QDateTime &cur) const
     const SectionNode &sn = sectionNode(s);
     switch (sn.type) {
     case TimeZoneSection:
-#if QT_CONFIG(timezone)
         return QTimeZone::MaxUtcOffsetSecs;
-#else
-        return +14 * 3600;  // NB: copied from QTimeZone
-#endif
     case Hour24Section:
     case Hour12Section:
         // This is special-cased in parseSection.
@@ -312,11 +266,7 @@ int QDateTimeParser::absoluteMin(int s) const
     const SectionNode &sn = sectionNode(s);
     switch (sn.type) {
     case TimeZoneSection:
-#if QT_CONFIG(timezone)
         return QTimeZone::MinUtcOffsetSecs;
-#else
-        return -14 * 3600;  // NB: copied from QTimeZone
-#endif
     case Hour24Section:
     case Hour12Section:
     case MinuteSection:
@@ -534,10 +484,11 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
 
             case 'z':
                 if (parserType != QMetaType::QDate) {
-                    const SectionNode sn = { MSecSection, i - add, countRepeat(newFormat, i, 3) < 3 ? 1 : 3, 0 };
+                    const int repeat = countRepeat(newFormat, i, 3);
+                    const SectionNode sn = { MSecSection, i - add, repeat < 3 ? 1 : 3, 0 };
                     newSectionNodes.append(sn);
                     appendSeparator(&newSeparators, newFormat, index, i - index, lastQuote);
-                    i += sn.count - 1;
+                    i += repeat - 1;
                     index = i + 1;
                     newDisplay |= MSecSection;
                 }
@@ -550,8 +501,7 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
                     appendSeparator(&newSeparators, newFormat, index, i - index, lastQuote);
                     newDisplay |= AmPmSection;
                     if (i + 1 < newFormat.size()
-                        && newFormat.sliced(i + 1).startsWith(QLatin1Char('p'),
-                                                              Qt::CaseInsensitive)) {
+                        && newFormat.sliced(i + 1).startsWith(u'p', Qt::CaseInsensitive)) {
                         ++i;
                         if (newFormat.at(i) != QLatin1Char(caseOpt == UpperCase ? 'P' : 'p'))
                             caseOpt = NativeCase;
@@ -600,10 +550,8 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
                 break;
             case 't':
                 if (parserType == QMetaType::QDateTime) {
-                    // TODO (in qlocale.cpp's serialization, too) QTBUG-95966:
-                    // decide what different lengths of 't' format should do,
-                    // instead of repetition !
-                    const SectionNode sn = { TimeZoneSection, i - add, 1, 0 };
+                    const SectionNode sn
+                        = { TimeZoneSection, i - add, countRepeat(newFormat, i, 4), 0 };
                     newSectionNodes.append(sn);
                     appendSeparator(&newSeparators, newFormat, index, i - index, lastQuote);
                     i += sn.count - 1;
@@ -644,7 +592,7 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
 //     }
 
     QDTPDEBUG << newFormat << displayFormat;
-    QDTPDEBUGN("separators:\n'%s'", separators.join(QLatin1String("\n")).toLatin1().constData());
+    QDTPDEBUGN("separators:\n'%s'", separators.join("\n"_L1).toLatin1().constData());
 
     return true;
 }
@@ -718,16 +666,12 @@ int QDateTimeParser::sectionMaxSize(Section s, int count) const
 
     case DayOfWeekSectionShort:
     case DayOfWeekSectionLong:
-#if !QT_CONFIG(textdate)
-        return 2;
-#else
+#if QT_CONFIG(textdate)
         mcount = 7;
         Q_FALLTHROUGH();
 #endif
     case MonthSection:
-#if !QT_CONFIG(textdate)
-        return 2;
-#else
+#if QT_CONFIG(textdate)
         if (count <= 2)
             return 2;
 
@@ -743,7 +687,9 @@ int QDateTimeParser::sectionMaxSize(Section s, int count) const
             }
             return ret;
         }
-#endif
+#else
+        return 2;
+#endif // textdate
     case MSecSection:
         return 3;
     case YearSection:
@@ -820,7 +766,7 @@ QDateTimeParser::parseSection(const QDateTime &currentValue, int sectionIndex, i
 
     const int sectionmaxsize = sectionMaxSize(sectionIndex);
     const bool negate = (sn.type == YearSection && m_text.size() > offset
-                         && m_text.at(offset) == QLatin1Char('-'));
+                         && m_text.at(offset) == u'-');
     const int negativeYearOffset = negate ? 1 : 0;
 
     // If the fields we've read thus far imply a time in a spring-forward,
@@ -865,7 +811,7 @@ QDateTimeParser::parseSection(const QDateTime &currentValue, int sectionIndex, i
     case TimeZoneSection:
         result = findTimeZone(sectionTextRef, defaultValue,
                               absoluteMax(sectionIndex),
-                              absoluteMin(sectionIndex));
+                              absoluteMin(sectionIndex), sn.count);
         break;
     case MonthSection:
     case DayOfWeekSectionShort:
@@ -902,8 +848,8 @@ QDateTimeParser::parseSection(const QDateTime &currentValue, int sectionIndex, i
     case MSecSection: {
         int used = negativeYearOffset;
         // We already sliced off the - sign if it was legitimately present.
-        if (sectionTextRef.startsWith(QLatin1Char('-'))
-            || sectionTextRef.startsWith(QLatin1Char('+'))) {
+        if (sectionTextRef.startsWith(u'-')
+            || sectionTextRef.startsWith(u'+')) {
             if (separators.at(sectionIndex + 1).startsWith(sectionTextRef[0]))
                 result = ParsedSection(Intermediate, 0, used);
             break;
@@ -972,7 +918,7 @@ QDateTimeParser::parseSection(const QDateTime &currentValue, int sectionIndex, i
                     if (skipToNextSection(sectionIndex, defaultValue, digitsStr)) {
                         const int missingZeroes = sectionmaxsize - digitsStr.size();
                         result = ParsedSection(Acceptable, last, sectionmaxsize, missingZeroes);
-                        m_text.insert(offset, QString(missingZeroes, QLatin1Char('0')));
+                        m_text.insert(offset, QString(missingZeroes, u'0'));
                         ++(const_cast<QDateTimeParser*>(this)->sectionNodes[sectionIndex].zeroesAdded);
                     } else {
                         result = ParsedSection(Intermediate, last, used);;
@@ -1008,11 +954,7 @@ static int weekDayWithinMonth(QCalendar calendar, int year, int month, int day,
     const int maxDay = calendar.daysInMonth(month, year); // 0 if no such month
     day = maxDay > 1 ? qBound(1, day, maxDay) : qMax(1, day);
     day += dayOfWeekDiff(weekDay, calendar.dayOfWeek(QDate(year, month, day, calendar)));
-    if (day <= 0)
-        return day + 7;
-    if (maxDay > 0 && day > maxDay)
-        return day - 7;
-    return day;
+    return day <= 0 ? day + 7 : maxDay > 0 && day > maxDay ? day - 7 : day;
 }
 
 /*!
@@ -1030,7 +972,7 @@ static QDate actualDate(QDateTimeParser::Sections known, const QCalendar &calend
     if (actual.isValid() && year % 100 == year2digits && calendar.dayOfWeek(actual) == dayofweek)
         return actual; // The obvious candidate is fine :-)
 
-    if (dayofweek < 1 || dayofweek > 7) // Invalid: ignore
+    if (dayofweek < 1 || dayofweek > 7) // Intercallary (or invalid): ignore
         known &= ~QDateTimeParser::DayOfWeekSectionMask;
 
     // Assuming year > 0 ...
@@ -1054,16 +996,21 @@ static QDate actualDate(QDateTimeParser::Sections known, const QCalendar &calend
     }
 
     QDate first(year, month, 1, calendar);
-    int last = known & QDateTimeParser::YearSection && known & QDateTimeParser::MonthSection
-        ? first.daysInMonth(calendar) : 0;
+    int last = known & QDateTimeParser::MonthSection
+        ? (known & QDateTimeParser::YearSection
+           ? calendar.daysInMonth(month, year) : calendar.daysInMonth(month))
+        : 0;
+    // We can only fix DOW if we know year as well as month (hence last):
+    const bool fixDayOfWeek = last && known & QDateTimeParser::YearSection
+            && known & QDateTimeParser::DayOfWeekSectionMask;
     // If we also know day-of-week, tweak last to the last in the month that matches it:
-    if (last && known & QDateTimeParser::DayOfWeekSectionMask) {
-        int diff = (dayofweek - calendar.dayOfWeek(first) - last) % 7;
+    if (fixDayOfWeek) {
+        const int diff = (dayofweek - calendar.dayOfWeek(first) - last) % 7;
         Q_ASSERT(diff <= 0); // C++11 specifies (-ve) % (+ve) to be <= 0.
         last += diff;
     }
     if (day < 1) {
-        if (known & QDateTimeParser::DayOfWeekSectionMask && last) {
+        if (fixDayOfWeek) {
             day = 1 + dayofweek - calendar.dayOfWeek(first);
             if (day < 1)
                 day += 7;
@@ -1071,7 +1018,7 @@ static QDate actualDate(QDateTimeParser::Sections known, const QCalendar &calend
             day = 1;
         }
         known &= ~QDateTimeParser::DaySection;
-    } else if (day > 31) {
+    } else if (day > calendar.maximumDaysInMonth()) {
         day = last;
         known &= ~QDateTimeParser::DaySection;
     } else if (last && day > last && (known & QDateTimeParser::DaySection) == 0) {
@@ -1215,26 +1162,7 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
     int second = defaultTime.second();
     int msec = defaultTime.msec();
     int dayofweek = calendar.dayOfWeek(defaultDate);
-    Qt::TimeSpec tspec = defaultValue.timeSpec();
-    int zoneOffset = 0; // In seconds; local - UTC
-#if QT_CONFIG(timezone)
-    QTimeZone timeZone;
-#endif
-    switch (tspec) {
-    case Qt::OffsetFromUTC: // timeZone is ignored
-        zoneOffset = defaultValue.offsetFromUtc();
-        break;
-#if QT_CONFIG(timezone)
-    case Qt::TimeZone:
-        timeZone = defaultValue.timeZone();
-        if (timeZone.isValid())
-            zoneOffset = timeZone.offsetFromUtc(defaultValue);
-        // else: is there anything we can do about this ?
-        break;
-#endif
-    default: // zoneOffset and timeZone are ignored
-        break;
-    }
+    QTimeZone timeZone = defaultValue.timeRepresentation();
 
     int ampm = -1;
     Sections isSet = NoSection;
@@ -1251,6 +1179,7 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
         pos += separator.size();
         sectionNodes[index].pos = pos;
         int *current = nullptr;
+        int zoneOffset; // Needed to serve as *current when setting zone
         const SectionNode sn = sectionNodes.at(index);
         ParsedSection sect;
 
@@ -1258,12 +1187,7 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
             const QDate date = actualDate(isSet, calendar, year, year2digits,
                                           month, day, dayofweek);
             const QTime time = actualTime(isSet, hour, hour12, ampm, minute, second, msec);
-            sect = parseSection(
-#if QT_CONFIG(timezone)
-                                tspec == Qt::TimeZone ? QDateTime(date, time, timeZone) :
-#endif
-                                QDateTime(date, time, tspec, zoneOffset),
-                                index, pos);
+            sect = parseSection(QDateTime(date, time, timeZone), index, pos);
         }
 
         QDTPDEBUG << "sectionValue" << sn.name() << m_text
@@ -1273,7 +1197,7 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
         if (fixup && sect.state == Intermediate && sect.used < sn.count) {
             const FieldInfo fi = fieldInfo(index);
             if ((fi & (Numeric|FixedWidth)) == (Numeric|FixedWidth)) {
-                const QString newText = QString::fromLatin1("%1").arg(sect.value, sn.count, 10, QLatin1Char('0'));
+                const QString newText = QString("%1"_L1).arg(sect.value, sn.count, 10, '0'_L1);
                 m_text.replace(pos, sect.used, newText);
                 sect.used = sn.count;
             }
@@ -1292,23 +1216,24 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
                 QStringView zoneName = QStringView{m_text}.sliced(pos, sect.used);
                 Q_ASSERT(!zoneName.isEmpty()); // sect.used > 0
 
-                const QStringView offsetStr = zoneName.startsWith(QLatin1String("UTC"))
-                                             ? zoneName.sliced(3) : zoneName;
-                const bool isUtcOffset = offsetStr.startsWith(QLatin1Char('+'))
-                                         || offsetStr.startsWith(QLatin1Char('-'));
-                const bool isUtc = zoneName == QLatin1String("Z")
-                                   || zoneName == QLatin1String("UTC");
+                const QStringView offsetStr
+                    = zoneName.startsWith("UTC"_L1) ? zoneName.sliced(3) : zoneName;
+                const bool isUtcOffset = offsetStr.startsWith(u'+') || offsetStr.startsWith(u'-');
+                const bool isUtc = zoneName == "Z"_L1 || zoneName == "UTC"_L1;
 
                 if (isUtc || isUtcOffset) {
-                    tspec = sect.value ? Qt::OffsetFromUTC : Qt::UTC;
+                    timeZone = QTimeZone::fromSecondsAheadOfUtc(sect.value);
                 } else {
 #if QT_CONFIG(timezone)
-                    timeZone = QTimeZone(zoneName.toLatin1());
-                    tspec = timeZone.isValid()
-                        ? Qt::TimeZone
-                        : (Q_ASSERT(startsWithLocalTimeZone(zoneName)), Qt::LocalTime);
+                    QTimeZone namedZone = QTimeZone(zoneName.toLatin1());
+                    if (namedZone.isValid()) {
+                        timeZone = namedZone;
+                    } else {
+                        Q_ASSERT(startsWithLocalTimeZone(zoneName));
+                        timeZone = QTimeZone::LocalTime;
+                    }
 #else
-                    tspec = Qt::LocalTime;
+                    timeZone = QTimeZone::LocalTime;
 #endif
                 }
             }
@@ -1450,11 +1375,7 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
 
     const QDate date(year, month, day, calendar);
     const QTime time(hour, minute, second, msec);
-    const QDateTime when =
-#if QT_CONFIG(timezone)
-            tspec == Qt::TimeZone ? QDateTime(date, time, timeZone) :
-#endif
-            QDateTime(date, time, tspec, zoneOffset);
+    const QDateTime when = QDateTime(date, time, timeZone);
 
     // If hour wasn't specified, check the default we're using exists on the
     // given date (which might be a spring-forward, skipping an hour).
@@ -1463,11 +1384,7 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
         case QMetaType::QDateTime: {
             qint64 msecs = when.toMSecsSinceEpoch();
             // Fortunately, that gets a useful answer, even though when is invalid ...
-            const QDateTime replace =
-#if QT_CONFIG(timezone)
-                tspec == Qt::TimeZone ? QDateTime::fromMSecsSinceEpoch(msecs, timeZone) :
-#endif
-                QDateTime::fromMSecsSinceEpoch(msecs, tspec, zoneOffset);
+            const QDateTime replace = QDateTime::fromMSecsSinceEpoch(msecs, timeZone);
             const QTime tick = replace.time();
             if (replace.date() == date
                 && (!(isSet & MinuteSection) || tick.minute() == minute)
@@ -1478,14 +1395,13 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
         } break;
         case QMetaType::QDate:
             // Don't care about time, so just use start of day (and ignore spec):
-            return StateNode(date.startOfDay(Qt::UTC), state, padding, conflicts);
+            return StateNode(date.startOfDay(QTimeZone::UTC), state, padding, conflicts);
             break;
         case QMetaType::QTime:
-            // Don't care about date or spec, so pick a safe spec:
-            return StateNode(QDateTime(date, time, Qt::UTC), state, padding, conflicts);
+            // Don't care about date or representation, so pick a safe representation:
+            return StateNode(QDateTime(date, time, QTimeZone::UTC), state, padding, conflicts);
         default:
-            Q_UNREACHABLE();
-            return StateNode();
+            Q_UNREACHABLE_RETURN(StateNode());
         }
     }
 
@@ -1507,7 +1423,7 @@ QDateTimeParser::parse(const QString &input, int position,
     QDTPDEBUG << "parse" << input;
     StateNode scan = scanString(defaultValue, fixup);
     QDTPDEBUGN("'%s' => '%s'(%s)", m_text.toLatin1().constData(),
-               scan.value.toString(QLatin1String("yyyy/MM/dd hh:mm:ss.zzz")).toLatin1().constData(),
+               scan.value.toString("yyyy/MM/dd hh:mm:ss.zzz"_L1).toLatin1().constData(),
                stateName(scan.state).toLatin1().constData());
 
     if (scan.value.isValid() && scan.state != Invalid) {
@@ -1738,24 +1654,29 @@ int QDateTimeParser::findDay(const QString &str1, int startDay, int sectionIndex
 
   Return's .value is UTC offset in seconds.
   The caller must verify that the offset is within a valid range.
+  The mode is 1 for permissive parsing, 2 and 3 for strict offset-only format
+  (no UTC prefix) with no colon for 2 and a colon for 3.
  */
-QDateTimeParser::ParsedSection QDateTimeParser::findUtcOffset(QStringView str) const
+QDateTimeParser::ParsedSection QDateTimeParser::findUtcOffset(QStringView str, int mode) const
 {
-    const bool startsWithUtc = str.startsWith(QLatin1String("UTC"));
-    // Get rid of UTC prefix if it exists
+    Q_ASSERT(mode > 0 && mode < 4);
+    const bool startsWithUtc = str.startsWith("UTC"_L1);
+    // Deal with UTC prefix if present:
     if (startsWithUtc) {
+        if (mode != 1)
+            return ParsedSection();
         str = str.sliced(3);
         if (str.isEmpty())
             return ParsedSection(Acceptable, 0, 3);
     }
 
-    const bool negativeSign = str.startsWith(QLatin1Char('-'));
+    const bool negativeSign = str.startsWith(u'-');
     // Must start with a sign:
-    if (!negativeSign && !str.startsWith(QLatin1Char('+')))
+    if (!negativeSign && !str.startsWith(u'+'))
         return ParsedSection();
     str = str.sliced(1);  // drop sign
 
-    const int colonPosition = str.indexOf(QLatin1Char(':'));
+    const int colonPosition = str.indexOf(u':');
     // Colon that belongs to offset is at most at position 2 (hh:mm)
     bool hasColon = (colonPosition >= 0 && colonPosition < 3);
 
@@ -1777,6 +1698,8 @@ QDateTimeParser::ParsedSection QDateTimeParser::findUtcOffset(QStringView str) c
         i = hoursLength;
         hasColon = false;
     }
+    if (mode == (hasColon ? 2 : 3))
+        return ParsedSection();
     str.truncate(i);  // The rest of the string is not part of the UTC offset
 
     bool isInt = false;
@@ -1833,7 +1756,7 @@ QDateTimeParser::findTimeZoneName(QStringView str, const QDateTime &when) const
     int count = 0;
     Q_ASSERT(index <= str.size());
     while (lastSlash < index) {
-        int slash = str.indexOf(QLatin1Char('/'), lastSlash + 1);
+        int slash = str.indexOf(u'/', lastSlash + 1);
         if (slash < 0 || slash > index)
             slash = index; // i.e. the end of the candidate text
         else if (++count > 5)
@@ -1861,17 +1784,26 @@ QDateTimeParser::findTimeZoneName(QStringView str, const QDateTime &when) const
 
   Return's .value is zone's offset, zone time - UTC time, in seconds.
   See QTimeZonePrivate::isValidId() for the format of zone names.
- */
+
+  The mode is the number of 't' characters in the field specifier:
+  * 1: any recognized format
+  * 2: only the simple offset format, without colon
+  * 3: only the simple offset format, with colon
+  * 4: only a zone name
+*/
 QDateTimeParser::ParsedSection
 QDateTimeParser::findTimeZone(QStringView str, const QDateTime &when,
-                              int maxVal, int minVal) const
+                              int maxVal, int minVal, int mode) const
 {
+    Q_ASSERT(mode > 0 && mode <= 4);
     // Short-cut Zulu suffix when it's all there is (rather than a prefix match):
-    if (str == QLatin1Char('Z'))
+    if (mode == 1 && str == u'Z')
         return ParsedSection(Acceptable, 0, 1);
 
-    ParsedSection section = findUtcOffset(str);
-    if (section.used <= 0)  // if nothing used, try time zone parsing
+    ParsedSection section;
+    if (mode != 4)
+        section = findUtcOffset(str, mode);
+    if (mode != 2 && mode != 3 && section.used <= 0)  // if nothing used, try time zone parsing
         section = findTimeZoneName(str, when);
     // It can be a well formed time zone specifier, but with value out of range
     if (section.state == Acceptable && (section.value < minVal || section.value > maxVal))
@@ -1879,11 +1811,13 @@ QDateTimeParser::findTimeZone(QStringView str, const QDateTime &when,
     if (section.used > 0)
         return section;
 
-    // Check if string is UTC or alias to UTC, after all other options
-    if (str.startsWith(QLatin1String("UTC")))
-        return ParsedSection(Acceptable, 0, 3);
-    if (str.startsWith(QLatin1Char('Z')))
-        return ParsedSection(Acceptable, 0, 1);
+    if (mode == 1) {
+        // Check if string is UTC or alias to UTC, after all other options
+        if (str.startsWith("UTC"_L1))
+            return ParsedSection(Acceptable, 0, 3);
+        if (str.startsWith(u'Z'))
+            return ParsedSection(Acceptable, 0, 1);
+    }
 
     return ParsedSection();
 }
@@ -2066,18 +2000,18 @@ QString QDateTimeParser::SectionNode::format() const
 {
     QChar fillChar;
     switch (type) {
-    case AmPmSection: return QLatin1String(count == 1 ? "ap" : count == 2 ? "AP" : "Ap");
-    case MSecSection: fillChar = QLatin1Char('z'); break;
-    case SecondSection: fillChar = QLatin1Char('s'); break;
-    case MinuteSection: fillChar = QLatin1Char('m'); break;
-    case Hour24Section: fillChar = QLatin1Char('H'); break;
-    case Hour12Section: fillChar = QLatin1Char('h'); break;
+    case AmPmSection: return count == 1 ? "ap"_L1 : count == 2 ? "AP"_L1 : "Ap"_L1;
+    case MSecSection: fillChar = u'z'; break;
+    case SecondSection: fillChar = u's'; break;
+    case MinuteSection: fillChar = u'm'; break;
+    case Hour24Section: fillChar = u'H'; break;
+    case Hour12Section: fillChar = u'h'; break;
     case DayOfWeekSectionShort:
     case DayOfWeekSectionLong:
-    case DaySection: fillChar = QLatin1Char('d'); break;
-    case MonthSection: fillChar = QLatin1Char('M'); break;
+    case DaySection: fillChar = u'd'; break;
+    case MonthSection: fillChar = u'M'; break;
     case YearSection2Digits:
-    case YearSection: fillChar = QLatin1Char('y'); break;
+    case YearSection: fillChar = u'y'; break;
     default:
         qWarning("QDateTimeParser::sectionFormat Internal error (%ls)",
                  qUtf16Printable(name(type)));
@@ -2175,23 +2109,23 @@ bool QDateTimeParser::skipToNextSection(int index, const QDateTime &current, QSt
 QString QDateTimeParser::SectionNode::name(QDateTimeParser::Section s)
 {
     switch (s) {
-    case QDateTimeParser::AmPmSection: return QLatin1String("AmPmSection");
-    case QDateTimeParser::DaySection: return QLatin1String("DaySection");
-    case QDateTimeParser::DayOfWeekSectionShort: return QLatin1String("DayOfWeekSectionShort");
-    case QDateTimeParser::DayOfWeekSectionLong: return QLatin1String("DayOfWeekSectionLong");
-    case QDateTimeParser::Hour24Section: return QLatin1String("Hour24Section");
-    case QDateTimeParser::Hour12Section: return QLatin1String("Hour12Section");
-    case QDateTimeParser::MSecSection: return QLatin1String("MSecSection");
-    case QDateTimeParser::MinuteSection: return QLatin1String("MinuteSection");
-    case QDateTimeParser::MonthSection: return QLatin1String("MonthSection");
-    case QDateTimeParser::SecondSection: return QLatin1String("SecondSection");
-    case QDateTimeParser::TimeZoneSection: return QLatin1String("TimeZoneSection");
-    case QDateTimeParser::YearSection: return QLatin1String("YearSection");
-    case QDateTimeParser::YearSection2Digits: return QLatin1String("YearSection2Digits");
-    case QDateTimeParser::NoSection: return QLatin1String("NoSection");
-    case QDateTimeParser::FirstSection: return QLatin1String("FirstSection");
-    case QDateTimeParser::LastSection: return QLatin1String("LastSection");
-    default: return QLatin1String("Unknown section ") + QString::number(int(s));
+    case QDateTimeParser::AmPmSection: return "AmPmSection"_L1;
+    case QDateTimeParser::DaySection: return "DaySection"_L1;
+    case QDateTimeParser::DayOfWeekSectionShort: return "DayOfWeekSectionShort"_L1;
+    case QDateTimeParser::DayOfWeekSectionLong: return "DayOfWeekSectionLong"_L1;
+    case QDateTimeParser::Hour24Section: return "Hour24Section"_L1;
+    case QDateTimeParser::Hour12Section: return "Hour12Section"_L1;
+    case QDateTimeParser::MSecSection: return "MSecSection"_L1;
+    case QDateTimeParser::MinuteSection: return "MinuteSection"_L1;
+    case QDateTimeParser::MonthSection: return "MonthSection"_L1;
+    case QDateTimeParser::SecondSection: return "SecondSection"_L1;
+    case QDateTimeParser::TimeZoneSection: return "TimeZoneSection"_L1;
+    case QDateTimeParser::YearSection: return "YearSection"_L1;
+    case QDateTimeParser::YearSection2Digits: return "YearSection2Digits"_L1;
+    case QDateTimeParser::NoSection: return "NoSection"_L1;
+    case QDateTimeParser::FirstSection: return "FirstSection"_L1;
+    case QDateTimeParser::LastSection: return "LastSection"_L1;
+    default: return "Unknown section "_L1 + QString::number(int(s));
     }
 }
 
@@ -2203,17 +2137,17 @@ QString QDateTimeParser::SectionNode::name(QDateTimeParser::Section s)
 QString QDateTimeParser::stateName(State s) const
 {
     switch (s) {
-    case Invalid: return QLatin1String("Invalid");
-    case Intermediate: return QLatin1String("Intermediate");
-    case Acceptable: return QLatin1String("Acceptable");
-    default: return QLatin1String("Unknown state ") + QString::number(s);
+    case Invalid: return "Invalid"_L1;
+    case Intermediate: return "Intermediate"_L1;
+    case Acceptable: return "Acceptable"_L1;
+    default: return "Unknown state "_L1 + QString::number(s);
     }
 }
 
 // Only called when we want only one of date or time; use UTC to avoid bogus DST issues.
 bool QDateTimeParser::fromString(const QString &t, QDate *date, QTime *time) const
 {
-    QDateTime val(QDate(1900, 1, 1).startOfDay(Qt::UTC));
+    QDateTime val(QDate(1900, 1, 1).startOfDay(QTimeZone::UTC));
     const StateNode tmp = parse(t, -1, val, false);
     if (tmp.state != Acceptable || tmp.conflicts)
         return false;
@@ -2253,7 +2187,7 @@ QDateTime QDateTimeParser::getMinimum() const
     //     method. At the time of writing, this is done by QDateTimeEditPrivate.
 
     // Cache the only case
-    static const QDateTime localTimeMin(QDATETIMEEDIT_DATE_MIN.startOfDay(Qt::LocalTime));
+    static const QDateTime localTimeMin(QDATETIMEEDIT_DATE_MIN.startOfDay());
     return localTimeMin;
 }
 
@@ -2264,7 +2198,7 @@ QDateTime QDateTimeParser::getMaximum() const
     //     method. At the time of writing, this is done by QDateTimeEditPrivate.
 
     // Cache the only case
-    static const QDateTime localTimeMax(QDATETIMEEDIT_DATE_MAX.endOfDay(Qt::LocalTime));
+    static const QDateTime localTimeMax(QDATETIMEEDIT_DATE_MAX.endOfDay());
     return localTimeMax;
 }
 
@@ -2278,8 +2212,7 @@ QString QDateTimeParser::getAmPmText(AmPm ap, Case cs) const
     case LowerCase: return raw.toLower();
     case NativeCase: return raw;
     }
-    Q_UNREACHABLE();
-    return raw;
+    Q_UNREACHABLE_RETURN(raw);
 }
 
 /*
