@@ -48,7 +48,7 @@ public:
 
 void QDBusMetaTypeId::init()
 {
-    static QBasicAtomicInt initialized = Q_BASIC_ATOMIC_INITIALIZER(false);
+    Q_CONSTINIT static QBasicAtomicInt initialized = Q_BASIC_ATOMIC_INITIALIZER(false);
 
     // reentrancy is not a problem since everything else is locked on their own
     // set the guard variable at the end
