@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Qt Gui module
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef QRHID3D11_P_H
 #define QRHID3D11_P_H
@@ -57,13 +21,14 @@
 #include <QWindow>
 
 #include <d3d11_1.h>
-#include <dxgi1_3.h>
+#include <dxgi1_6.h>
+#include <dcomp.h>
 
 QT_BEGIN_NAMESPACE
 
 struct QD3D11Buffer : public QRhiBuffer
 {
-    QD3D11Buffer(QRhiImplementation *rhi, Type type, UsageFlags usage, int size);
+    QD3D11Buffer(QRhiImplementation *rhi, Type type, UsageFlags usage, quint32 size);
     ~QD3D11Buffer();
     void destroy() override;
     bool create() override;
@@ -71,12 +36,12 @@ struct QD3D11Buffer : public QRhiBuffer
     char *beginFullDynamicBufferUpdateForCurrentFrame() override;
     void endFullDynamicBufferUpdateForCurrentFrame() override;
 
-    ID3D11UnorderedAccessView *unorderedAccessView();
+    ID3D11UnorderedAccessView *unorderedAccessView(quint32 offset);
 
     ID3D11Buffer *buffer = nullptr;
     char *dynBuf = nullptr;
     bool hasPendingDynamicUpdates = false;
-    ID3D11UnorderedAccessView *uav = nullptr;
+    QHash<quint32, ID3D11UnorderedAccessView *> uavs;
     uint generation = 0;
     friend class QRhiD3D11;
 };
@@ -96,13 +61,14 @@ struct QD3D11RenderBuffer : public QRhiRenderBuffer
     ID3D11RenderTargetView *rtv = nullptr;
     DXGI_FORMAT dxgiFormat;
     DXGI_SAMPLE_DESC sampleDesc;
+    uint generation = 0;
     friend class QRhiD3D11;
 };
 
 struct QD3D11Texture : public QRhiTexture
 {
-    QD3D11Texture(QRhiImplementation *rhi, Format format, const QSize &pixelSize,
-                  int sampleCount, Flags flags);
+    QD3D11Texture(QRhiImplementation *rhi, Format format, const QSize &pixelSize, int depth,
+                  int arraySize, int sampleCount, Flags flags);
     ~QD3D11Texture();
     void destroy() override;
     bool create() override;
@@ -112,14 +78,24 @@ struct QD3D11Texture : public QRhiTexture
     bool prepareCreate(QSize *adjustedSize = nullptr);
     bool finishCreate();
     ID3D11UnorderedAccessView *unorderedAccessViewForLevel(int level);
+    ID3D11Resource *textureResource() const
+    {
+        if (tex)
+            return tex;
+        else if (tex1D)
+            return tex1D;
+        return tex3D;
+    }
 
     ID3D11Texture2D *tex = nullptr;
+    ID3D11Texture3D *tex3D = nullptr;
+    ID3D11Texture1D *tex1D = nullptr;
     bool owns = true;
     ID3D11ShaderResourceView *srv = nullptr;
     DXGI_FORMAT dxgiFormat;
     uint mipLevelCount = 0;
     DXGI_SAMPLE_DESC sampleDesc;
-    ID3D11UnorderedAccessView *perLevelViews[QRhi::MAX_LEVELS];
+    ID3D11UnorderedAccessView *perLevelViews[QRhi::MAX_MIP_LEVELS];
     uint generation = 0;
     friend class QRhiD3D11;
 };
@@ -144,6 +120,7 @@ struct QD3D11RenderPassDescriptor : public QRhiRenderPassDescriptor
     void destroy() override;
     bool isCompatible(const QRhiRenderPassDescriptor *other) const override;
     QRhiRenderPassDescriptor *newCompatibleRenderPassDescriptor() const override;
+    QVector<quint32> serializedFormat() const override;
 };
 
 struct QD3D11RenderTargetData
@@ -164,12 +141,14 @@ struct QD3D11RenderTargetData
     static const int MAX_COLOR_ATTACHMENTS = 8;
     ID3D11RenderTargetView *rtv[MAX_COLOR_ATTACHMENTS];
     ID3D11DepthStencilView *dsv = nullptr;
+
+    QRhiRenderTargetAttachmentTracker::ResIdList currentResIdList;
 };
 
-struct QD3D11ReferenceRenderTarget : public QRhiRenderTarget
+struct QD3D11SwapChainRenderTarget : public QRhiSwapChainRenderTarget
 {
-    QD3D11ReferenceRenderTarget(QRhiImplementation *rhi);
-    ~QD3D11ReferenceRenderTarget();
+    QD3D11SwapChainRenderTarget(QRhiImplementation *rhi, QRhiSwapChain *swapchain);
+    ~QD3D11SwapChainRenderTarget();
     void destroy() override;
 
     QSize pixelSize() const override;
@@ -206,6 +185,7 @@ struct QD3D11ShaderResourceBindings : public QRhiShaderResourceBindings
     ~QD3D11ShaderResourceBindings();
     void destroy() override;
     bool create() override;
+    void updateResources(UpdateFlags flags) override;
 
     bool hasDynamicOffset = false;
     QVarLengthArray<QRhiShaderResourceBinding, 8> sortedBindings;
@@ -244,39 +224,66 @@ struct QD3D11ShaderResourceBindings : public QRhiShaderResourceBindings
     };
     QVarLengthArray<BoundResourceData, 8> boundResourceData;
 
-    bool vsubufsPresent = false;
-    bool fsubufsPresent = false;
-    bool csubufsPresent = false;
-    bool vssamplersPresent = false;
-    bool fssamplersPresent = false;
-    bool cssamplersPresent = false;
-    bool csUAVsPresent = false;
+    struct StageUniformBufferBatches {
+        bool present = false;
+        QRhiBatchedBindings<ID3D11Buffer *> ubufs;
+        QRhiBatchedBindings<UINT> ubuforigbindings;
+        QRhiBatchedBindings<UINT> ubufoffsets;
+        QRhiBatchedBindings<UINT> ubufsizes;
+        void finish() {
+            present = ubufs.finish();
+            ubuforigbindings.finish();
+            ubufoffsets.finish();
+            ubufsizes.finish();
+        }
+        void clear() {
+            ubufs.clear();
+            ubuforigbindings.clear();
+            ubufoffsets.clear();
+            ubufsizes.clear();
+        }
+    };
 
-    QRhiBatchedBindings<ID3D11Buffer *> vsubufs;
-    QRhiBatchedBindings<UINT> vsubuforigbindings;
-    QRhiBatchedBindings<UINT> vsubufoffsets;
-    QRhiBatchedBindings<UINT> vsubufsizes;
+    struct StageSamplerBatches {
+        bool present = false;
+        QRhiBatchedBindings<ID3D11SamplerState *> samplers;
+        QRhiBatchedBindings<ID3D11ShaderResourceView *> shaderresources;
+        void finish() {
+            present = samplers.finish();
+            shaderresources.finish();
+        }
+        void clear() {
+            samplers.clear();
+            shaderresources.clear();
+        }
+    };
 
-    QRhiBatchedBindings<ID3D11Buffer *> fsubufs;
-    QRhiBatchedBindings<UINT> fsubuforigbindings;
-    QRhiBatchedBindings<UINT> fsubufoffsets;
-    QRhiBatchedBindings<UINT> fsubufsizes;
+    struct StageUavBatches {
+        bool present = false;
+        QRhiBatchedBindings<ID3D11UnorderedAccessView *> uavs;
+        void finish() {
+            present = uavs.finish();
+        }
+        void clear() {
+            uavs.clear();
+        }
+    };
 
-    QRhiBatchedBindings<ID3D11Buffer *> csubufs;
-    QRhiBatchedBindings<UINT> csubuforigbindings;
-    QRhiBatchedBindings<UINT> csubufoffsets;
-    QRhiBatchedBindings<UINT> csubufsizes;
+    StageUniformBufferBatches vsUniformBufferBatches;
+    StageUniformBufferBatches hsUniformBufferBatches;
+    StageUniformBufferBatches dsUniformBufferBatches;
+    StageUniformBufferBatches gsUniformBufferBatches;
+    StageUniformBufferBatches fsUniformBufferBatches;
+    StageUniformBufferBatches csUniformBufferBatches;
 
-    QRhiBatchedBindings<ID3D11SamplerState *> vssamplers;
-    QRhiBatchedBindings<ID3D11ShaderResourceView *> vsshaderresources;
+    StageSamplerBatches vsSamplerBatches;
+    StageSamplerBatches hsSamplerBatches;
+    StageSamplerBatches dsSamplerBatches;
+    StageSamplerBatches gsSamplerBatches;
+    StageSamplerBatches fsSamplerBatches;
+    StageSamplerBatches csSamplerBatches;
 
-    QRhiBatchedBindings<ID3D11SamplerState *> fssamplers;
-    QRhiBatchedBindings<ID3D11ShaderResourceView *> fsshaderresources;
-
-    QRhiBatchedBindings<ID3D11SamplerState *> cssamplers;
-    QRhiBatchedBindings<ID3D11ShaderResourceView *> csshaderresources;
-
-    QRhiBatchedBindings<ID3D11UnorderedAccessView *> csUAVs;
+    StageUavBatches csUavBatches;
 
     friend class QRhiD3D11;
 };
@@ -296,6 +303,18 @@ struct QD3D11GraphicsPipeline : public QRhiGraphicsPipeline
         ID3D11VertexShader *shader = nullptr;
         QShader::NativeResourceBindingMap nativeResourceBindingMap;
     } vs;
+    struct {
+        ID3D11HullShader *shader = nullptr;
+        QShader::NativeResourceBindingMap nativeResourceBindingMap;
+    } hs;
+    struct {
+        ID3D11DomainShader *shader = nullptr;
+        QShader::NativeResourceBindingMap nativeResourceBindingMap;
+    } ds;
+    struct {
+        ID3D11GeometryShader *shader = nullptr;
+        QShader::NativeResourceBindingMap nativeResourceBindingMap;
+    } gs;
     struct {
         ID3D11PixelShader *shader = nullptr;
         QShader::NativeResourceBindingMap nativeResourceBindingMap;
@@ -439,6 +458,7 @@ struct QD3D11CommandBuffer : public QRhiCommandBuffer
                 UINT dstSubRes;
                 UINT dstX;
                 UINT dstY;
+                UINT dstZ;
                 ID3D11Resource *src;
                 UINT srcSubRes;
                 bool hasSrcBox;
@@ -522,9 +542,6 @@ struct QD3D11CommandBuffer : public QRhiCommandBuffer
         currentGraphicsPipeline = nullptr;
         currentComputePipeline = nullptr;
         currentPipelineGeneration = 0;
-        resetCachedShaderResourceState();
-    }
-    void resetCachedShaderResourceState() {
         currentGraphicsSrb = nullptr;
         currentComputeSrb = nullptr;
         currentSrbGeneration = 0;
@@ -546,6 +563,8 @@ struct QD3D11SwapChain : public QRhiSwapChain
     QRhiRenderTarget *currentFrameRenderTarget() override;
 
     QSize surfacePixelSize() override;
+    bool isFormatSupported(Format f) override;
+    QRhiSwapChainHdrInfo hdrInfo() override;
 
     QRhiRenderPassDescriptor *newCompatibleRenderPassDescriptor() override;
     bool createOrResize() override;
@@ -556,10 +575,12 @@ struct QD3D11SwapChain : public QRhiSwapChain
 
     QWindow *window = nullptr;
     QSize pixelSize;
-    QD3D11ReferenceRenderTarget rt;
+    QD3D11SwapChainRenderTarget rt;
     QD3D11CommandBuffer cb;
     DXGI_FORMAT colorFormat;
+    DXGI_FORMAT srgbAdjustedColorFormat;
     IDXGISwapChain *swapChain = nullptr;
+    UINT swapChainFlags = 0;
     static const int BUFFER_COUNT = 2;
     ID3D11Texture2D *backBufferTex;
     ID3D11RenderTargetView *backBufferRtv;
@@ -573,6 +594,8 @@ struct QD3D11SwapChain : public QRhiSwapChain
     ID3D11Query *timestampDisjointQuery[BUFFER_COUNT];
     ID3D11Query *timestampQuery[BUFFER_COUNT * 2];
     UINT swapInterval = 1;
+    IDCompositionTarget *dcompTarget = nullptr;
+    IDCompositionVisual *dcompVisual = nullptr;
 };
 
 class QRhiD3D11 : public QRhiImplementation
@@ -588,7 +611,7 @@ public:
     QRhiShaderResourceBindings *createShaderResourceBindings() override;
     QRhiBuffer *createBuffer(QRhiBuffer::Type type,
                              QRhiBuffer::UsageFlags usage,
-                             int size) override;
+                             quint32 size) override;
     QRhiRenderBuffer *createRenderBuffer(QRhiRenderBuffer::Type type,
                                          const QSize &pixelSize,
                                          int sampleCount,
@@ -596,6 +619,8 @@ public:
                                          QRhiTexture::Format backingFormatHint) override;
     QRhiTexture *createTexture(QRhiTexture::Format format,
                                const QSize &pixelSize,
+                               int depth,
+                               int arraySize,
                                int sampleCount,
                                QRhiTexture::Flags flags) override;
     QRhiSampler *createSampler(QRhiSampler::Filter magFilter,
@@ -676,7 +701,7 @@ public:
     int resourceLimit(QRhi::ResourceLimit limit) const override;
     const QRhiNativeHandles *nativeHandles() override;
     QRhiDriverInfo driverInfo() const override;
-    void sendVMemStatsToProfiler() override;
+    QRhiStats statistics() override;
     bool makeThreadLocalNativeContextCurrent() override;
     void releaseCachedResources() override;
     bool isDeviceLost() const override;
@@ -699,7 +724,11 @@ public:
     void finishActiveReadbacks();
     void reportLiveObjects(ID3D11Device *device);
     void clearShaderCache();
+    QByteArray compileHlslShaderSource(const QShader &shader, QShader::Variant shaderVariant, uint flags,
+                                       QString *error, QShaderKey *usedShaderKey);
+    bool ensureDirectCompositionDevice();
 
+    QRhi::Flags rhiFlags;
     bool debugLayer = false;
     bool importedDeviceAndContext = false;
     ID3D11Device *dev = nullptr;
@@ -707,9 +736,10 @@ public:
     D3D_FEATURE_LEVEL featureLevel = D3D_FEATURE_LEVEL(0);
     LUID adapterLuid = {};
     ID3DUserDefinedAnnotation *annotations = nullptr;
+    IDXGIAdapter1 *activeAdapter = nullptr;
     IDXGIFactory1 *dxgiFactory = nullptr;
-    bool hasDxgi2 = false;
-    bool supportsFlipDiscardSwapchain = false;
+    IDCompositionDevice *dcompDevice = nullptr;
+    bool supportsAllowTearing = false;
     bool deviceLost = false;
     QRhiD3D11NativeHandles nativeHandlesStruct;
     QRhiDriverInfo driverInfoStruct;
@@ -718,6 +748,9 @@ public:
         int vsHighestActiveVertexBufferBinding = -1;
         bool vsHasIndexBufferBound = false;
         int vsHighestActiveSrvBinding = -1;
+        int hsHighestActiveSrvBinding = -1;
+        int dsHighestActiveSrvBinding = -1;
+        int gsHighestActiveSrvBinding = -1;
         int fsHighestActiveSrvBinding = -1;
         int csHighestActiveSrvBinding = -1;
         int csHighestActiveUavBinding = -1;
@@ -769,11 +802,45 @@ public:
         void releaseResources();
         void activate();
     } deviceCurse;
+
+    // This is what gets exposed as the "pipeline cache", not that that concept
+    // applies anyway. Here we are just storing the DX bytecode for a shader so
+    // we can skip the HLSL->DXBC compilation when the QShader has HLSL source
+    // code and the same shader source has already been compiled before.
+    // m_shaderCache seemingly does the same, but this here does not care about
+    // the ID3D11*Shader, this is just about the bytecode and about allowing
+    // the data to be serialized to persistent storage and then reloaded in
+    // future runs of the app, or when creating another QRhi, etc.
+    struct BytecodeCacheKey {
+        QByteArray sourceHash;
+        QByteArray target;
+        QByteArray entryPoint;
+        uint compileFlags;
+    };
+    QHash<BytecodeCacheKey, QByteArray> m_bytecodeCache;
 };
 
 Q_DECLARE_TYPEINFO(QRhiD3D11::TextureReadback, Q_RELOCATABLE_TYPE);
 Q_DECLARE_TYPEINFO(QRhiD3D11::BufferReadback, Q_RELOCATABLE_TYPE);
 
+inline bool operator==(const QRhiD3D11::BytecodeCacheKey &a, const QRhiD3D11::BytecodeCacheKey &b) noexcept
+{
+    return a.sourceHash == b.sourceHash
+            && a.target == b.target
+            && a.entryPoint == b.entryPoint
+            && a.compileFlags == b.compileFlags;
+}
+
+inline bool operator!=(const QRhiD3D11::BytecodeCacheKey &a, const QRhiD3D11::BytecodeCacheKey &b) noexcept
+{
+    return !(a == b);
+}
+
+inline size_t qHash(const QRhiD3D11::BytecodeCacheKey &k, size_t seed = 0) noexcept
+{
+    return qHash(k.sourceHash, seed) ^ qHash(k.target) ^ qHash(k.entryPoint) ^ k.compileFlags;
+}
+
 QT_END_NAMESPACE
 
 #endif
