@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "private/qlayoutengine_p.h"
 #if QT_CONFIG(itemviews)
@@ -57,7 +21,7 @@
 #include "qwhatsthis.h"
 #endif
 #include "private/qtextengine_p.h"
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
 #include "qaccessible.h"
 #endif
 #ifdef Q_OS_MACOS
@@ -70,6 +34,8 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 namespace {
 class CloseButton : public QAbstractButton
 {
@@ -406,12 +372,12 @@ void QTabBarPrivate::init()
 {
     Q_Q(QTabBar);
     leftB = new QToolButton(q);
-    leftB->setObjectName(u"ScrollLeftButton"_qs);
+    leftB->setObjectName(u"ScrollLeftButton"_s);
     leftB->setAutoRepeat(true);
     QObject::connect(leftB, SIGNAL(clicked()), q, SLOT(_q_scrollTabs()));
     leftB->hide();
     rightB = new QToolButton(q);
-    rightB->setObjectName(u"ScrollRightButton"_qs);
+    rightB->setObjectName(u"ScrollRightButton"_s);
     rightB->setAutoRepeat(true);
     QObject::connect(rightB, SIGNAL(clicked()), q, SLOT(_q_scrollTabs()));
     rightB->hide();
@@ -424,7 +390,7 @@ void QTabBarPrivate::init()
 #endif
         q->setFocusPolicy(Qt::TabFocus);
 
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
     leftB->setAccessibleName(QTabBar::tr("Scroll Left"));
     rightB->setAccessibleName(QTabBar::tr("Scroll Right"));
 #endif
@@ -438,7 +404,7 @@ int QTabBarPrivate::indexAtPos(const QPoint &p) const
     Q_Q(const QTabBar);
     if (q->tabRect(currentIndex).contains(p))
         return currentIndex;
-    for (int i = 0; i < tabList.count(); ++i)
+    for (int i = 0; i < tabList.size(); ++i)
         if (tabList.at(i)->enabled && q->tabRect(i).contains(p))
             return i;
     return -1;
@@ -456,7 +422,7 @@ void QTabBarPrivate::layoutTabs()
     int hiddenTabs = 0;
 
     Qt::Alignment tabAlignment = Qt::Alignment(q->style()->styleHint(QStyle::SH_TabBar_Alignment, nullptr, q));
-    QList<QLayoutStruct> tabChain(tabList.count() + 2);
+    QList<QLayoutStruct> tabChain(tabList.size() + 2);
 
     // We put an empty item at the front and back and set its expansive attribute
     // depending on tabAlignment and expanding.
@@ -479,7 +445,7 @@ void QTabBarPrivate::layoutTabs()
         int minx = 0;
         int x = 0;
         int maxHeight = 0;
-        for (int i = 0; i < tabList.count(); ++i) {
+        for (int i = 0; i < tabList.size(); ++i) {
             const auto tab = tabList.at(i);
             if (!tab->visible) {
                 ++hiddenTabs;
@@ -510,7 +476,7 @@ void QTabBarPrivate::layoutTabs()
         int miny = 0;
         int y = 0;
         int maxWidth = 0;
-        for (int i = 0; i < tabList.count(); ++i) {
+        for (int i = 0; i < tabList.size(); ++i) {
             auto tab = tabList.at(i);
             if (!tab->visible) {
                 ++hiddenTabs;
@@ -545,14 +511,14 @@ void QTabBarPrivate::layoutTabs()
                                         && (tabAlignment != Qt::AlignRight)
                                         && (tabAlignment != Qt::AlignJustify);
     tabChain[tabChainIndex].empty = true;
-    Q_ASSERT(tabChainIndex == tabChain.count() - 1 - hiddenTabs); // add an assert just to make sure.
+    Q_ASSERT(tabChainIndex == tabChain.size() - 1 - hiddenTabs); // add an assert just to make sure.
 
     // Do the calculation
-    qGeomCalc(tabChain, 0, tabChain.count(), 0, qMax(available, last), 0);
+    qGeomCalc(tabChain, 0, tabChain.size(), 0, qMax(available, last), 0);
 
     // Use the results
     hiddenTabs = 0;
-    for (int i = 0; i < tabList.count(); ++i) {
+    for (int i = 0; i < tabList.size(); ++i) {
         auto tab = tabList.at(i);
         if (!tab->visible) {
             tab->rect = QRect();
@@ -566,9 +532,8 @@ void QTabBarPrivate::layoutTabs()
             tab->rect.setRect(0, lstruct.pos, maxExtent, lstruct.size);
     }
 
-    if (useScrollButtons && tabList.count() && last > available) {
+    if (useScrollButtons && tabList.size() && last > available) {
         const QRect scrollRect = normalizedScrollRect(0);
-        scrollOffset = -scrollRect.left();
 
         Q_Q(QTabBar);
         QStyleOption opt;
@@ -605,10 +570,9 @@ void QTabBarPrivate::layoutTabs()
         leftB->show();
 
         rightB->setGeometry(scrollButtonRightRect);
-        rightB->setEnabled(last - scrollOffset > scrollRect.x() + scrollRect.width());
+        rightB->setEnabled(last + scrollRect.left() > scrollRect.x() + scrollRect.width());
         rightB->show();
     } else {
-        scrollOffset = 0;
         rightB->hide();
         leftB->hide();
     }
@@ -625,6 +589,11 @@ QRect QTabBarPrivate::normalizedScrollRect(int index)
     // tab bar itself is in a different orientation.
 
     Q_Q(QTabBar);
+    // If scrollbuttons are not visible, then there's no tear either, and
+    // the entire widget is the scroll rect.
+    if (leftB->isHidden())
+        return verticalTabs(shape) ? q->rect().transposed() : q->rect();
+
     QStyleOptionTab opt;
     q->initStyleOption(&opt, currentIndex);
     opt.rect = q->rect();
@@ -704,16 +673,18 @@ int QTabBarPrivate::hoveredTabIndex() const
 void QTabBarPrivate::makeVisible(int index)
 {
     Q_Q(QTabBar);
-    if (!validIndex(index) || leftB->isHidden())
+    if (!validIndex(index))
         return;
 
     const QRect tabRect = tabList.at(index)->rect;
     const int oldScrollOffset = scrollOffset;
     const bool horiz = !verticalTabs(shape);
+    const int available = horiz ? q->width() : q->height();
     const int tabStart = horiz ? tabRect.left() : tabRect.top();
     const int tabEnd = horiz ? tabRect.right() : tabRect.bottom();
     const int lastTabEnd = horiz ? tabList.constLast()->rect.right() : tabList.constLast()->rect.bottom();
     const QRect scrollRect = normalizedScrollRect(index);
+    const QRect entireScrollRect = normalizedScrollRect(0); // ignore tears
     const int scrolledTabBarStart = qMax(1, scrollRect.left() + scrollOffset);
     const int scrolledTabBarEnd = qMin(lastTabEnd - 1, scrollRect.right() + scrollOffset);
 
@@ -722,7 +693,13 @@ void QTabBarPrivate::makeVisible(int index)
         scrollOffset = tabStart - scrollRect.left();
     } else if (tabEnd > scrolledTabBarEnd) {
         // Tab is outside on the right, so scroll right.
-        scrollOffset = tabEnd - scrollRect.right();
+        scrollOffset = qMax(0, tabEnd - scrollRect.right());
+    } else if (scrollOffset + entireScrollRect.width() > lastTabEnd + 1) {
+        // fill any free space on the right without overshooting
+        scrollOffset = qMax(0, lastTabEnd - entireScrollRect.width() + 1);
+    } else if (available >= lastTabEnd) {
+        // the entire tabbar fits, reset scroll
+        scrollOffset = 0;
     }
 
     leftB->setEnabled(scrollOffset > -scrollRect.left());
@@ -802,7 +779,7 @@ void QTabBarPrivate::_q_closeTab()
     QObject *object = q->sender();
     int tabToClose = -1;
     QTabBar::ButtonPosition closeSide = (QTabBar::ButtonPosition)q->style()->styleHint(QStyle::SH_TabBar_CloseButtonPosition, nullptr, q);
-    for (int i = 0; i < tabList.count(); ++i) {
+    for (int i = 0; i < tabList.size(); ++i) {
         if (closeSide == QTabBar::LeftSide) {
             if (tabList.at(i)->leftWidget == object) {
                 tabToClose = i;
@@ -829,7 +806,7 @@ void QTabBarPrivate::_q_scrollTabs()
     int i = -1;
 
     if (sender == leftB) {
-        for (i = tabList.count() - 1; i >= 0; --i) {
+        for (i = tabList.size() - 1; i >= 0; --i) {
             int start = horizontal ? tabList.at(i)->rect.left() : tabList.at(i)->rect.top();
             if (start < scrollRect.left()) {
                 makeVisible(i);
@@ -837,7 +814,7 @@ void QTabBarPrivate::_q_scrollTabs()
             }
         }
     } else if (sender == rightB) {
-        for (i = 0; i < tabList.count(); ++i) {
+        for (i = 0; i < tabList.size(); ++i) {
             const auto tabRect = tabList.at(i)->rect;
             int start = horizontal ? tabRect.left() : tabRect.top();
             int end = horizontal ? tabRect.right() : tabRect.bottom();
@@ -960,7 +937,7 @@ int QTabBar::addTab(const QIcon& icon, const QString &text)
 
 /*!
     Inserts a new tab with text \a text at position \a index. If \a
-    index is out of range, the new tab is appened. Returns the new
+    index is out of range, the new tab is appended. Returns the new
     tab's index.
 */
 int QTabBar::insertTab(int index, const QString &text)
@@ -984,7 +961,7 @@ int QTabBar::insertTab(int index, const QIcon& icon, const QString &text)
 {
     Q_D(QTabBar);
     if (!d->validIndex(index)) {
-        index = d->tabList.count();
+        index = d->tabList.size();
         d->tabList.append(new QTabBarPrivate::Tab(icon, text));
     } else {
         d->tabList.insert(index, new QTabBarPrivate::Tab(icon, text));
@@ -994,7 +971,7 @@ int QTabBar::insertTab(int index, const QIcon& icon, const QString &text)
 #endif
     d->firstVisible = qMax(qMin(index, d->firstVisible), 0);
     d->refresh();
-    if (d->tabList.count() == 1)
+    if (d->tabList.size() == 1)
         setCurrentIndex(index);
     else if (index <= d->currentIndex)
         ++d->currentIndex;
@@ -1013,7 +990,7 @@ int QTabBar::insertTab(int index, const QIcon& icon, const QString &text)
         setTabButton(index, closeSide, closeButton);
     }
 
-    for (const auto tab : qAsConst(d->tabList)) {
+    for (const auto tab : std::as_const(d->tabList)) {
         if (tab->lastTab >= index)
             ++tab->lastTab;
     }
@@ -1059,7 +1036,7 @@ void QTabBar::removeTab(int index)
         int newIndex = removedTab->lastTab;
         d->tabList.removeAt(index);
         delete removedTab;
-        for (auto tab : qAsConst(d->tabList)) {
+        for (auto tab : std::as_const(d->tabList)) {
             if (tab->lastTab == index)
                 tab->lastTab = -1;
             if (tab->lastTab > index)
@@ -1439,7 +1416,7 @@ void QTabBar::setCurrentIndex(int index)
         // If the size hint depends on whether the tab is selected (for instance a style
         // sheet rule that sets a bold font on the 'selected' tab) then we need to
         // re-layout the entire tab bar. To minimize the cost, do that only if the
-        // size hint changes for the tab that becomes the current tab (the old curent tab
+        // size hint changes for the tab that becomes the current tab (the old current tab
         // will most certainly do the same). QTBUG-6905
         if (tabRect(index).size() != tabSizeHint(index))
             d->layoutTabs();
@@ -1450,7 +1427,7 @@ void QTabBar::setCurrentIndex(int index)
             d->layoutTab(oldIndex);
         }
         d->layoutTab(index);
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
         if (QAccessible::isActive()) {
             if (hasFocus()) {
                 QAccessibleEvent focusEvent(this, QAccessible::Focus);
@@ -1503,7 +1480,7 @@ void QTabBar::setIconSize(const QSize &size)
 int QTabBar::count() const
 {
     Q_D(const QTabBar);
-    return d->tabList.count();
+    return d->tabList.size();
 }
 
 
@@ -1546,10 +1523,10 @@ QSize QTabBar::minimumSizeHint() const
 // Compute the most-elided possible text, for minimumSizeHint
 static QString computeElidedText(Qt::TextElideMode mode, const QString &text)
 {
-    if (text.length() <= 3)
+    if (text.size() <= 3)
         return text;
 
-    static const QLatin1String Ellipses("...");
+    static const auto Ellipses = "..."_L1;
     QString ret;
     switch (mode) {
     case Qt::ElideRight:
@@ -1679,6 +1656,8 @@ void QTabBar::showEvent(QShowEvent *)
         d->refresh();
     if (!d->validIndex(d->currentIndex))
         setCurrentIndex(0);
+    else
+        d->makeVisible(d->currentIndex);
     d->updateMacBorderMetrics();
 }
 
@@ -1755,7 +1734,7 @@ bool QTabBar::event(QEvent *event)
 
     case QEvent::Shortcut: {
         QShortcutEvent *se = static_cast<QShortcutEvent *>(event);
-        for (int i = 0; i < d->tabList.count(); ++i) {
+        for (int i = 0; i < d->tabList.size(); ++i) {
             const QTabBarPrivate::Tab *tab = d->tabList.at(i);
             if (tab->shortcutId == se->shortcutId()) {
                 setCurrentIndex(i);
@@ -1812,6 +1791,8 @@ void QTabBar::resizeEvent(QResizeEvent *)
     Q_D(QTabBar);
     if (d->layoutDirty)
         updateGeometry();
+
+    // when resizing, we want to keep the scroll offset as much as possible
     d->layoutTabs();
 
     d->makeVisible(d->currentIndex);
@@ -1838,7 +1819,7 @@ void QTabBar::paintEvent(QPaintEvent *)
         selected = d->pressedIndex;
     const QRect scrollRect = d->normalizedScrollRect();
 
-    for (int i = 0; i < d->tabList.count(); ++i)
+    for (int i = 0; i < d->tabList.size(); ++i)
          optTabBase.tabBarRect |= tabRect(i);
 
     optTabBase.selectedTabRect = tabRect(selected);
@@ -1860,7 +1841,7 @@ void QTabBar::paintEvent(QPaintEvent *)
             p.setClipRegion(QRegion(rect()) - buttonRegion);
     }
 
-    for (int i = 0; i < d->tabList.count(); ++i) {
+    for (int i = 0; i < d->tabList.size(); ++i) {
         const auto tab = d->tabList.at(i);
         if (!tab->visible)
             continue;
@@ -1950,7 +1931,7 @@ void QTabBarPrivate::calculateFirstLastVisible(int index, bool visible, bool rem
     } else {
         if (remove || (index == firstVisible)) {
             firstVisible = -1;
-            for (int i = 0; i < tabList.count(); ++i) {
+            for (int i = 0; i < tabList.size(); ++i) {
                 if (tabList.at(i)->visible) {
                     firstVisible = i;
                     break;
@@ -1959,7 +1940,7 @@ void QTabBarPrivate::calculateFirstLastVisible(int index, bool visible, bool rem
         }
         if (remove || (index == lastVisible)) {
             lastVisible = -1;
-            for (int i = tabList.count() - 1; i >= 0; --i) {
+            for (int i = tabList.size() - 1; i >= 0; --i) {
                 if (tabList.at(i)->visible) {
                     lastVisible = i;
                     break;
@@ -1977,7 +1958,7 @@ void QTabBarPrivate::calculateFirstLastVisible(int index, bool visible, bool rem
 int QTabBarPrivate::selectNewCurrentIndexFrom(int fromIndex)
 {
     int newindex = -1;
-    for (int i = fromIndex; i < tabList.count(); ++i) {
+    for (int i = fromIndex; i < tabList.size(); ++i) {
         if (at(i)->visible && at(i)->enabled) {
           newindex = i;
           break;
@@ -2073,7 +2054,7 @@ void QTabBar::moveTab(int from, int to)
     d->tabList.move(from, to);
 
     // update lastTab locations
-    for (const auto tab : qAsConst(d->tabList))
+    for (const auto tab : std::as_const(d->tabList))
         tab->lastTab = d->calculateNewPosition(from, to, tab->lastTab);
 
     // update external variables
@@ -2228,7 +2209,8 @@ void QTabBar::mouseMoveEvent(QMouseEvent *event)
                 }
             }
             // Buttons needs to follow the dragged tab
-            d->layoutTab(d->pressedIndex);
+            if (d->pressedIndex != -1)
+                d->layoutTab(d->pressedIndex);
 
             update();
         }
@@ -2291,7 +2273,7 @@ void QTabBarPrivate::moveTabFinished(int index)
     bool cleanup = (pressedIndex == index) || (pressedIndex == -1) || !validIndex(index);
     bool allAnimationsFinished = true;
 #if QT_CONFIG(animation)
-    for (const auto tab : qAsConst(tabList)) {
+    for (const auto tab : std::as_const(tabList)) {
         if (tab->animation && tab->animation->state() == QAbstractAnimation::Running) {
             allAnimationsFinished = false;
             break;
@@ -2301,7 +2283,7 @@ void QTabBarPrivate::moveTabFinished(int index)
     if (allAnimationsFinished && cleanup) {
         if (movingTab)
             movingTab->setVisible(false); // We might not get a mouse release
-        for (auto tab : qAsConst(tabList)) {
+        for (auto tab : std::as_const(tabList)) {
             tab->dragOffset = 0;
         }
         if (pressedIndex != -1 && movable) {
@@ -2579,7 +2561,7 @@ void QTabBar::setTabsClosable(bool closable)
     d->closeButtonOnTabs = closable;
     ButtonPosition closeSide = (ButtonPosition)style()->styleHint(QStyle::SH_TabBar_CloseButtonPosition, nullptr, this);
     if (!closable) {
-        for (auto tab : qAsConst(d->tabList)) {
+        for (auto tab : std::as_const(d->tabList)) {
             if (closeSide == LeftSide && tab->leftWidget) {
                 tab->leftWidget->deleteLater();
                 tab->leftWidget = nullptr;
@@ -2591,7 +2573,7 @@ void QTabBar::setTabsClosable(bool closable)
         }
     } else {
         bool newButtons = false;
-        for (int i = 0; i < d->tabList.count(); ++i) {
+        for (int i = 0; i < d->tabList.size(); ++i) {
             if (tabButton(i, closeSide))
                 continue;
             newButtons = true;
@@ -2786,10 +2768,11 @@ void QTabBar::setChangeCurrentOnDrag(bool change)
 
 /*!
     Sets \a widget on the tab \a index.  The widget is placed
-    on the left or right hand side depending upon the \a position.
+    on the left or right hand side depending on the \a position.
     \since 4.5
 
-    Any previously set widget in \a position is hidden.
+    Any previously set widget in \a position is hidden. Setting \a widget
+    to \nullptr will hide the current widget at \a position.
 
     The tab bar will take ownership of the widget and so all widgets set here
     will be deleted by the tab bar when it is destroyed unless you separately
@@ -2800,7 +2783,7 @@ void QTabBar::setChangeCurrentOnDrag(bool change)
 void QTabBar::setTabButton(int index, ButtonPosition position, QWidget *widget)
 {
     Q_D(QTabBar);
-    if (index < 0 || index >= d->tabList.count())
+    if (index < 0 || index >= d->tabList.size())
         return;
     if (widget) {
         widget->setParent(this);
@@ -2837,7 +2820,7 @@ QWidget *QTabBar::tabButton(int index, ButtonPosition position) const
     return nullptr;
 }
 
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
 /*!
     Sets the accessibleName of the tab at position \a index to \a name.
 */
@@ -2863,7 +2846,7 @@ QString QTabBar::accessibleTabName(int index) const
         return tab->accessibleName;
     return QString();
 }
-#endif // QT_NO_ACCESSIBILITY
+#endif // QT_CONFIG(accessibility)
 
 CloseButton::CloseButton(QWidget *parent)
     : QAbstractButton(parent)
